# -*- coding: utf-8 -*-
"""Top-level package for Granular phobos."""
from engine.grain import Engine 

__author__ = """Sid Gupta, Sagar Verma"""
__email__ = 'sid@granular.ai, sagar@granular.ai'
__version__ = 'v0.2.0'

__all__ = ["Engine"]